/**
 * Copyright (c) 2018-19, NVIDIA CORPORATION.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#pragma once

#include "opg/matrix/data.hpp"
#include "opg/matrix/part_descriptor.hpp"
#include <linalg/norm.h>
#include <cuml/common/cuml_allocator.hpp>
#include <linalg/cublas_wrappers.h>

namespace MLCommon {
namespace LinAlg {
namespace opg {

/**
 * @brief performs MNMG Least squares calculation.
 */
void colNorm2(Matrix::Data<double> &out,
		const std::vector<Matrix::Data<double>*> &in,
		const Matrix::PartDescriptor& inDesc,
		const cumlCommunicator& comm,
		const std::shared_ptr<deviceAllocator> allocator,
		cudaStream_t *streams,
		int n_streams,
		cublasHandle_t cublas_h);

void colNorm2(Matrix::Data<float> &out,
		const std::vector<Matrix::Data<float>*> &in,
		const Matrix::PartDescriptor& inDesc,
		const cumlCommunicator& comm,
		const std::shared_ptr<deviceAllocator> allocator,
		cudaStream_t *streams,
		int n_streams,
		cublasHandle_t cublas_h);

void colNorm2NoSeq(Matrix::Data<double> &out,
		const std::vector<Matrix::Data<double>*> &in,
		const Matrix::PartDescriptor& inDesc,
		const cumlCommunicator& comm,
		const std::shared_ptr<deviceAllocator> allocator,
		cudaStream_t *streams,
		int n_streams,
		cublasHandle_t cublas_h);

void colNorm2NoSeq(Matrix::Data<float> &out,
		const std::vector<Matrix::Data<float>*> &in,
		const Matrix::PartDescriptor& inDesc,
		const cumlCommunicator& comm,
		const std::shared_ptr<deviceAllocator> allocator,
		cudaStream_t *streams,
		int n_streams,
		cublasHandle_t cublas_h);

/** @} */

} // end namespace opg
} // end namespace LinAlg
} // end namespace MLCommon
