/*
* Copyright 1993-2019 NVIDIA Corporation.  All rights reserved.
*
* NOTICE TO LICENSEE:
*
* This source code and/or documentation ("Licensed Deliverables") are
* subject to NVIDIA intellectual property rights under U.S. and
* international Copyright laws.
*
* These Licensed Deliverables contained herein is PROPRIETARY and
* CONFIDENTIAL to NVIDIA and is being provided under the terms and
* conditions of a form of NVIDIA software license agreement by and
* between NVIDIA and Licensee ("License Agreement") or electronically
* accepted by Licensee.  Notwithstanding any terms or conditions to
* the contrary in the License Agreement, reproduction or disclosure
* of the Licensed Deliverables to any third party without the express
* written consent of NVIDIA is prohibited.
*
* NOTWITHSTANDING ANY TERMS OR CONDITIONS TO THE CONTRARY IN THE
* LICENSE AGREEMENT, NVIDIA MAKES NO REPRESENTATION ABOUT THE
* SUITABILITY OF THESE LICENSED DELIVERABLES FOR ANY PURPOSE.  IT IS
* PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY OF ANY KIND.
* NVIDIA DISCLAIMS ALL WARRANTIES WITH REGARD TO THESE LICENSED
* DELIVERABLES, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY,
* NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE.
* NOTWITHSTANDING ANY TERMS OR CONDITIONS TO THE CONTRARY IN THE
* LICENSE AGREEMENT, IN NO EVENT SHALL NVIDIA BE LIABLE FOR ANY
* SPECIAL, INDIRECT, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, OR ANY
* DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
* WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
* ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE
* OF THESE LICENSED DELIVERABLES.
*
* U.S. Government End Users.  These Licensed Deliverables are a
* "commercial item" as that term is defined at 48 C.F.R. 2.101 (OCT
* 1995), consisting of "commercial computer software" and "commercial
* computer software documentation" as such terms are used in 48
* C.F.R. 12.212 (SEPT 1995) and is provided to the U.S. Government
* only as a commercial end item.  Consistent with 48 C.F.R.12.212 and
* 48 C.F.R. 227.7202-1 through 227.7202-4 (JUNE 1995), all
* U.S. Government End Users acquire the Licensed Deliverables with
* only those rights set forth herein.
*
* Any use of the Licensed Deliverables in individual and commercial
* software must include, in the user documentation and internal
* comments to the code, the above Disclaimer and U.S. Government End
* Users Notice.
*/


#pragma once

#include <spmg/utils_spmg.hpp>

namespace MLCommon {
namespace LinAlg {

    /**
     * @defgroup ScalarOps Scalar operations on the input buffer
     * @param out the output buffer
     * @param in the input buffer
     * @param scalar the scalar used in the operations
     * @param len number of elements in the input buffer
     * @{
     */
    void addScalarSPMG(TypeSPMG<float> *out, TypeSPMG<float> *in, float scalar,
        int *gpu_ids, cudaStream_t *streams, int n_gpus, bool sync);
    void addScalarSPMG(TypeSPMG<double> *out, TypeSPMG<double> *in, double scalar,
        int *gpu_ids, cudaStream_t *streams, int n_gpus, bool sync);

    /**
     * @defgroup BinaryOps Element-wise binary operations on the input buffers
     * @param out the output buffer
     * @param in1 the first input buffer
     * @param in2 the second input buffer
     * @param len number of elements in the input buffers
     * @param n_gpus number of gpus
     * @{
     */
    void addSPMG(TypeSPMG<float> *out, TypeSPMG<float> *in1, TypeSPMG<float> *in2,
        int *gpu_ids, cudaStream_t *streams, int n_gpus, bool sync);
    void addSPMG(TypeSPMG<double> *out, TypeSPMG<double> *in1, TypeSPMG<double> *in2,
        int *gpu_ids, cudaStream_t *streams, int n_gpus, bool sync);


    /**
     * @defgroup BinaryOps Element-wise binary operations on the input buffers
     * @param out the output buffer
     * @param in1 the first input buffer
     * @param in2 the second input buffer
     * @param len number of elements in the input buffers
     * @param n_gpus number of gpus
     * @{
     */
    void scalarMultiplySPMG(TypeSPMG<float> *out, TypeSPMG<float> *in, float scalar,
        int *gpu_ids, cudaStream_t *streams, int n_gpus, bool sync);
    void scalarMultiplySPMG(TypeSPMG<double> *out, TypeSPMG<double> *in, double scalar,
        int *gpu_ids, cudaStream_t *streams, int n_gpus, bool sync);

    /**
     * @defgroup BinaryOps Element-wise binary operations on the input buffers
     * @param out the output buffer
     * @param in1 the first input buffer
     * @param in2 the second input buffer
     * @param len number of elements in the input buffers
     * @param n_gpus number of gpus
     * @{
     */
    void gemm_spmg(TypeSPMG<float> *A, int n_rows_a, int n_cols_a,
               TypeSPMG<float> *B, int n_rows_b, int n_cols_b,
               TypeSPMG<float> *C, int n_rows_c, int n_cols_c,
               bool trans_a,
               bool trans_b,
               const float alpha,
               const float beta,
               int *gpu_ids,
               int n_gpus,
               bool rowMajor);
    void gemm_spmg(TypeSPMG<double> *A, int n_rows_a, int n_cols_a,
               TypeSPMG<double> *B, int n_rows_b, int n_cols_b,
               TypeSPMG<double> *C, int n_rows_c, int n_cols_c,
               bool trans_a,
               bool trans_b,
               const double alpha,
               const double beta,
               int *gpu_ids,
               int n_gpus,
               bool rowMajor);

}
}
